from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.lap_times_lap_status_item import LapTimesLapStatusItem
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.duration import Duration


T = TypeVar("T", bound="LapTimesLap")


@_attrs_define
class LapTimesLap:
    """
    Attributes:
        lap (int | Unset):
        lead_lap (int | Unset):
        time_of_day (Duration | Unset):
        lap_time (Duration | Unset):
        difference_with_last_lap (Duration | Unset):
        difference_with_best_lap (Duration | Unset):
        speed (float | Unset):
        section_times (list[Duration] | Unset):
        in_pit (bool | Unset):
        status (list[LapTimesLapStatusItem] | Unset):
    """

    lap: int | Unset = UNSET
    lead_lap: int | Unset = UNSET
    time_of_day: Duration | Unset = UNSET
    lap_time: Duration | Unset = UNSET
    difference_with_last_lap: Duration | Unset = UNSET
    difference_with_best_lap: Duration | Unset = UNSET
    speed: float | Unset = UNSET
    section_times: list[Duration] | Unset = UNSET
    in_pit: bool | Unset = UNSET
    status: list[LapTimesLapStatusItem] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        lap = self.lap

        lead_lap = self.lead_lap

        time_of_day: dict[str, Any] | Unset = UNSET
        if not isinstance(self.time_of_day, Unset):
            time_of_day = self.time_of_day.to_dict()

        lap_time: dict[str, Any] | Unset = UNSET
        if not isinstance(self.lap_time, Unset):
            lap_time = self.lap_time.to_dict()

        difference_with_last_lap: dict[str, Any] | Unset = UNSET
        if not isinstance(self.difference_with_last_lap, Unset):
            difference_with_last_lap = self.difference_with_last_lap.to_dict()

        difference_with_best_lap: dict[str, Any] | Unset = UNSET
        if not isinstance(self.difference_with_best_lap, Unset):
            difference_with_best_lap = self.difference_with_best_lap.to_dict()

        speed = self.speed

        section_times: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.section_times, Unset):
            section_times = []
            for section_times_item_data in self.section_times:
                section_times_item = section_times_item_data.to_dict()
                section_times.append(section_times_item)

        in_pit = self.in_pit

        status: list[str] | Unset = UNSET
        if not isinstance(self.status, Unset):
            status = []
            for status_item_data in self.status:
                status_item = status_item_data.value
                status.append(status_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if lap is not UNSET:
            field_dict["lap"] = lap
        if lead_lap is not UNSET:
            field_dict["leadLap"] = lead_lap
        if time_of_day is not UNSET:
            field_dict["timeOfDay"] = time_of_day
        if lap_time is not UNSET:
            field_dict["lapTime"] = lap_time
        if difference_with_last_lap is not UNSET:
            field_dict["differenceWithLastLap"] = difference_with_last_lap
        if difference_with_best_lap is not UNSET:
            field_dict["differenceWithBestLap"] = difference_with_best_lap
        if speed is not UNSET:
            field_dict["speed"] = speed
        if section_times is not UNSET:
            field_dict["sectionTimes"] = section_times
        if in_pit is not UNSET:
            field_dict["inPit"] = in_pit
        if status is not UNSET:
            field_dict["status"] = status

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.duration import Duration

        d = dict(src_dict)
        lap = d.pop("lap", UNSET)

        lead_lap = d.pop("leadLap", UNSET)

        _time_of_day = d.pop("timeOfDay", UNSET)
        time_of_day: Duration | Unset
        if isinstance(_time_of_day, Unset):
            time_of_day = UNSET
        else:
            time_of_day = Duration.from_dict(_time_of_day)

        _lap_time = d.pop("lapTime", UNSET)
        lap_time: Duration | Unset
        if isinstance(_lap_time, Unset):
            lap_time = UNSET
        else:
            lap_time = Duration.from_dict(_lap_time)

        _difference_with_last_lap = d.pop("differenceWithLastLap", UNSET)
        difference_with_last_lap: Duration | Unset
        if isinstance(_difference_with_last_lap, Unset):
            difference_with_last_lap = UNSET
        else:
            difference_with_last_lap = Duration.from_dict(_difference_with_last_lap)

        _difference_with_best_lap = d.pop("differenceWithBestLap", UNSET)
        difference_with_best_lap: Duration | Unset
        if isinstance(_difference_with_best_lap, Unset):
            difference_with_best_lap = UNSET
        else:
            difference_with_best_lap = Duration.from_dict(_difference_with_best_lap)

        speed = d.pop("speed", UNSET)

        _section_times = d.pop("sectionTimes", UNSET)
        section_times: list[Duration] | Unset = UNSET
        if _section_times is not UNSET:
            section_times = []
            for section_times_item_data in _section_times:
                section_times_item = Duration.from_dict(section_times_item_data)

                section_times.append(section_times_item)

        in_pit = d.pop("inPit", UNSET)

        _status = d.pop("status", UNSET)
        status: list[LapTimesLapStatusItem] | Unset = UNSET
        if _status is not UNSET:
            status = []
            for status_item_data in _status:
                status_item = LapTimesLapStatusItem(status_item_data)

                status.append(status_item)

        lap_times_lap = cls(
            lap=lap,
            lead_lap=lead_lap,
            time_of_day=time_of_day,
            lap_time=lap_time,
            difference_with_last_lap=difference_with_last_lap,
            difference_with_best_lap=difference_with_best_lap,
            speed=speed,
            section_times=section_times,
            in_pit=in_pit,
            status=status,
        )

        lap_times_lap.additional_properties = d
        return lap_times_lap

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.group import Group
    from ..models.session import Session


T = TypeVar("T", bound="EventSessionsAndGroups")


@_attrs_define
class EventSessionsAndGroups:
    """
    Attributes:
        sessions (list[Session] | Unset):
        groups (list[Group] | Unset):
    """

    sessions: list[Session] | Unset = UNSET
    groups: list[Group] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        sessions: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.sessions, Unset):
            sessions = []
            for sessions_item_data in self.sessions:
                sessions_item = sessions_item_data.to_dict()
                sessions.append(sessions_item)

        groups: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.groups, Unset):
            groups = []
            for groups_item_data in self.groups:
                groups_item = groups_item_data.to_dict()
                groups.append(groups_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if sessions is not UNSET:
            field_dict["sessions"] = sessions
        if groups is not UNSET:
            field_dict["groups"] = groups

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.group import Group
        from ..models.session import Session

        d = dict(src_dict)
        _sessions = d.pop("sessions", UNSET)
        sessions: list[Session] | Unset = UNSET
        if _sessions is not UNSET:
            sessions = []
            for sessions_item_data in _sessions:
                sessions_item = Session.from_dict(sessions_item_data)

                sessions.append(sessions_item)

        _groups = d.pop("groups", UNSET)
        groups: list[Group] | Unset = UNSET
        if _groups is not UNSET:
            groups = []
            for groups_item_data in _groups:
                groups_item = Group.from_dict(groups_item_data)

                groups.append(groups_item)

        event_sessions_and_groups = cls(
            sessions=sessions,
            groups=groups,
        )

        event_sessions_and_groups.additional_properties = d
        return event_sessions_and_groups

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""Interactive CLI to run processing tasks on the exporter `output/` directory.

Features:
- Scan `output/full_dump/` for org IDs and let the user pick one (or `all`).
- Run session/lap/announcement extractors and import to SQLite.
- Non-interactive flags to run everything for a given org.

Usage (interactive):
  python examples/processing/processor_cli.py

Usage (non-interactive):
  python examples/processing/processor_cli.py --org 30476 --run-all --output output/full_dump/30476
"""
from __future__ import annotations

import argparse
import os
import subprocess
import sys
from pathlib import Path
from typing import List, Optional


HERE = Path(__file__).resolve().parent


def find_org_dirs(output_root: Path) -> List[Path]:
    p = output_root.expanduser()
    if not p.exists():
        return []
    return [d for d in sorted(p.iterdir()) if d.is_dir()]


def run_script(script: Path, args: List[str]) -> int:
    cmd = [sys.executable, str(script)] + args
    print(f"-> Running: {' '.join(cmd)}")
    return subprocess.run(cmd).returncode


def process_all(org_dir: Path, do_sqlite: bool = True) -> None:
    # locate files
    laps = org_dir / "laps.ndjson.gz"
    sessions = org_dir / "sessions.ndjson.gz"
    announcements = org_dir / "announcements.ndjson.gz"

    scripts_dir = HERE
    if laps.exists():
        rc = run_script(scripts_dir / "extract_laps_to_csv.py", ["--input", str(org_dir), "--out", str(org_dir / "laps_flat.csv")])
        if rc != 0:
            print("extract_laps_to_csv.py failed", rc)
    else:
        print("No laps.ndjson.gz found in", org_dir)

    if sessions.exists():
        rc = run_script(scripts_dir / "extract_sessions_to_csv.py", ["--input", str(org_dir), "--out", str(org_dir / "sessions_flat.csv")])
        if rc != 0:
            print("extract_sessions_to_csv.py failed", rc)
    else:
        print("No sessions.ndjson.gz found in", org_dir)

    if announcements.exists():
        rc = run_script(scripts_dir / "extract_announcements_to_csv.py", ["--input", str(org_dir), "--out", str(org_dir / "announcements_flat.csv")])
        if rc != 0:
            print("extract_announcements_to_csv.py failed", rc)
    else:
        print("No announcements.ndjson.gz found in", org_dir)

    if do_sqlite and laps.exists():
        rc = run_script(scripts_dir / "ndjson_to_sqlite.py", ["--input", str(org_dir / "laps.ndjson.gz"), "--out", str(org_dir / "dump.db")])
        if rc != 0:
            print("ndjson_to_sqlite.py failed", rc)


def interactive(output_root: Path) -> None:
    orgs = find_org_dirs(output_root)
    if not orgs:
        print("No org directories found under", output_root)
        return
    print("Found org directories:")
    for i, d in enumerate(orgs, start=1):
        print(f"  {i}) {d.name}")
    print("  a) all")
    choice = input("Select org number (or 'a' for all): ").strip()
    if choice.lower().startswith("a"):
        for d in orgs:
            print(f"Processing {d}")
            process_all(d)
    else:
        try:
            idx = int(choice) - 1
            d = orgs[idx]
            process_all(d)
        except Exception as exc:
            print("Invalid selection:", exc)


def main(argv: Optional[List[str]] = None) -> int:
    p = argparse.ArgumentParser(description="Processing CLI: run extractors on output/full_dump/<org>")
    p.add_argument("--output-root", type=Path, default=Path("output/full_dump"), help="Root directory containing org output dirs")
    p.add_argument("--org", type=str, help="Org ID directory to process (e.g. 30476)")
    p.add_argument("--run-all", action="store_true", help="Run all processing steps (laps/sessions/announcements/import)")
    p.add_argument("--no-sqlite", action="store_true", help="Skip SQLite import step")
    args = p.parse_args(argv)

    outroot = args.output_root
    if args.org:
        orgdir = outroot / args.org
        if not orgdir.exists():
            print("Org directory not found:", orgdir)
            return 2
        if args.run_all:
            process_all(orgdir, do_sqlite=not args.no_sqlite)
        else:
            # interactive per-org choices
            print("Processing directory:", orgdir)
            process_all(orgdir, do_sqlite=not args.no_sqlite)
        return 0

    # no org specified -> interactive selection
    interactive(outroot)
    return 0


if __name__ == "__main__":
    raise SystemExit(main())



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.championship_competitor_event_round_dto import ChampionshipCompetitorEventRoundDto


T = TypeVar("T", bound="ChampionshipCompetitorEventDto")


@_attrs_define
class ChampionshipCompetitorEventDto:
    """
    Attributes:
        event_id (int | Unset):
        points (float | Unset):
        position (str | Unset):
        is_dropped (bool | Unset):
        rounds (list[ChampionshipCompetitorEventRoundDto] | Unset):
    """

    event_id: int | Unset = UNSET
    points: float | Unset = UNSET
    position: str | Unset = UNSET
    is_dropped: bool | Unset = UNSET
    rounds: list[ChampionshipCompetitorEventRoundDto] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        event_id = self.event_id

        points = self.points

        position = self.position

        is_dropped = self.is_dropped

        rounds: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.rounds, Unset):
            rounds = []
            for rounds_item_data in self.rounds:
                rounds_item = rounds_item_data.to_dict()
                rounds.append(rounds_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if event_id is not UNSET:
            field_dict["eventId"] = event_id
        if points is not UNSET:
            field_dict["points"] = points
        if position is not UNSET:
            field_dict["position"] = position
        if is_dropped is not UNSET:
            field_dict["isDropped"] = is_dropped
        if rounds is not UNSET:
            field_dict["rounds"] = rounds

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.championship_competitor_event_round_dto import ChampionshipCompetitorEventRoundDto

        d = dict(src_dict)
        event_id = d.pop("eventId", UNSET)

        points = d.pop("points", UNSET)

        position = d.pop("position", UNSET)

        is_dropped = d.pop("isDropped", UNSET)

        _rounds = d.pop("rounds", UNSET)
        rounds: list[ChampionshipCompetitorEventRoundDto] | Unset = UNSET
        if _rounds is not UNSET:
            rounds = []
            for rounds_item_data in _rounds:
                rounds_item = ChampionshipCompetitorEventRoundDto.from_dict(rounds_item_data)

                rounds.append(rounds_item)

        championship_competitor_event_dto = cls(
            event_id=event_id,
            points=points,
            position=position,
            is_dropped=is_dropped,
            rounds=rounds,
        )

        championship_competitor_event_dto.additional_properties = d
        return championship_competitor_event_dto

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
